/*
*This code has been generated by OpenAI.com, on March 2th 2023, answering to my following request:
*
*"code for ESP8266 that should connect to a wi-Fi network, blink a led while connecting, 
*and start pinging www.google.com every 40 seconds. If there is no connection for 5 minutes 
*or if no answer after 7 ping consecutives attempts, a relay will be powered on for 15 
*seconds before the cycle restarts. The LED on pin D4 will also indicate the Wi-Fi connection status."
*
*The code worked right away. I only had to make small corrections, to the lines that manages the 
*switching on the LED and the relays, to change in HIGH, where was indicated in LOW, and vice versa, 
*to be able to adapt the code to the scheme I adopted to connect the ESP8266 board to the relays.
*
*by P.L. 3_2023 - giano2002
*/


#include <ESP8266WiFi.h>

const char* ssid = "************"; // Replace with your Wi-Fi network SSID
const char* password = "*****************"; // Replace with your Wi-Fi network password

const int ledPin = D4; // Pin for the LED
const int relayPin = D7; // Pin for the relay

const int pingInterval = 40; // Ping interval in seconds
const int maxPingFails = 7; // Maximum consecutive ping failures before triggering the relay
const int relayDelay = 15000; // Time in milliseconds to keep the relay powered on

int pingFails = 0; // Counter for consecutive ping failures
unsigned long lastPingTime = 0; // Time of the last ping
bool wifiConnected = false; // Flag indicating if Wi-Fi is connected or not

WiFiClient client; // Create a client object for pinging Google

void setup() {
  pinMode(ledPin, OUTPUT);
  pinMode(relayPin, OUTPUT);
  digitalWrite(relayPin, LOW); // Turn off the relay initially
  
  Serial.begin(115200);
  Serial.println("Connecting to Wi-Fi...");
  
  // Connect to Wi-Fi network
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    digitalWrite(ledPin, !digitalRead(ledPin)); // Blink the LED while connecting
    delay(500);
    Serial.print(".");
  }
  
  wifiConnected = true;
  digitalWrite(ledPin, LOW); // Turn on the LED when connected
  Serial.println("\nConnected to Wi-Fi");
}

void loop() {
  if (wifiConnected) {
    if (millis() - lastPingTime >= pingInterval * 1000) { // Time to ping Google
      if (client.connect("www.google.com", 80)) { // Ping successful
        Serial.println("Ping successful");
        pingFails = 0;
        digitalWrite(ledPin, LOW); // Turn on the LED when ping is successful
        client.stop();
      } else { // Ping failed
        Serial.println("Ping failed");
        pingFails++;
        digitalWrite(ledPin, HIGH); // Turn off the LED when ping fails
      }
      
      lastPingTime = millis(); // Update the last ping time
      
      if (pingFails >= maxPingFails || millis() - lastPingTime >= 5 * 60 * 1000) { // Time to trigger the relay
        Serial.println("Triggering the relay");
        digitalWrite(relayPin, HIGH); // Power on the relay
        digitalWrite(ledPin, HIGH); // Turn off the LED when triggering the relay
        delay(relayDelay); // Keep the relay powered on for relayDelay milliseconds
        digitalWrite(relayPin, LOW); // Power off the relay
        pingFails = 0; // Reset the ping fails counter
        lastPingTime = millis(); // Reset the last ping time
        digitalWrite(ledPin, LOW); // Turn on the LED after triggering the relay
      }
    }
  } else { // Wi-Fi not connected
    digitalWrite(ledPin, !digitalRead(ledPin)); // Blink the LED to indicate Wi-Fi not connected
    delay(500);
    WiFi.begin(ssid, password); // Try to connect to Wi-Fi again
    if (WiFi.status() == WL_CONNECTED) {
      wifiConnected = true;
      digitalWrite(ledPin, LOW); // Turn on the LED when connected
      Serial.println("\nConnected to Wi-Fi");
    }
  }
}